<?php
namespace Controller;

use Util\OpensslCrypt;
use Util\Utilities;
use Zeedhi\Framework\DTO;
use Service\Manager\Mongo\Manager;
use Service\DeleteCascadeService;
use Service\ConnectionService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Util\Environment;

class QueryRunner extends Crud {
    protected $connectionService;
	protected $environment;
    public function __construct(ConnectionService $connectionService, Environment $environment) {
		$this->connectionService = $connectionService;
		$this->environment = $environment;
	}
    
    public function runQuery(DTO\Request\Row $request, DTO\Response $response){
    	ini_set('max_execution_time','600');
        set_time_limit(600);
        $row = $request->getRow();
		$organizationID = Utilities::getOrganizationIdForMongo();
		$clientRoom = $this->connectionService->getClientRoom($organizationID, $row);
        $dbUrl = '';
		if(isset($row['connectionDbUrl'])){
			$dbUrl = $row['connectionDbUrl'];
		} else if(isset($connectionParams['connectionHost']) && isset($connectionParams['connectionPort']) && isset($connectionParams['connectionDbname'])){
			$dbUrl = "jdbc:oracle:thin:@".$row['connectionHost'].":".$row['connectionPort']."/".$row['connectionDbname'];
		}
        $driverClass = isset($row['driverClass'])? $row['driverClass'] : null;

        if($driverClass == "Spreadsheet"){
            $row['query'] = $row['query']."::::EXEC";
        }
		$connectionsParams = array(
			'dbUrlSsl' => OpensslCrypt::encrypt($dbUrl),
			'usernameSsl' => OpensslCrypt::encrypt($row['connectionUsername']),
			'password' => $row['connectionPassword'],
        	'passwordSsl' => isset($row['connectionPasswordSsl']) ? $row['connectionPasswordSsl'] : null,
			'query' => $row['query'],
            'driverClass' => $driverClass,
            'dialect' =>isset($row['dialect'])? $row['dialect'] : null,
            'oracleWalletUse' =>isset($row['oracleWalletUse'])? $row['oracleWalletUse'] : null,
			"room" => $clientRoom
		);
		
        try {
            if ($connectionsParams['oracleWalletUse'] == 'Y') {
                $wallet = Utilities::getWallet($connectionsParams);
                if ( Utilities::getOrganizationUseWallet($wallet)) {
                    $queryResult = Utilities::execQueryWallet($connectionsParams['query'],
                        'queryRunner', $wallet);
                } else {
                    $queryResult = $this->connectionService->executeQuery($connectionsParams, 120);
                }
            } else {
                $queryResult = $this->connectionService->executeQuery($connectionsParams, 120);
            }
			$response->addDataSet(new DataSet('queryResult', $queryResult));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 0));
        }
    }
}