<?php

namespace Controller;

use Util\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Service\Traits\TResponse;
use Service\zmartAccessProfile as ZmartAccessProfileService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;


class ZmartAccessProfile{

    use TResponse;
    
    protected $dataSourceName = "zmartAccessProfile";

    /** @var ZmartAccessProfileService $accessProfileService */
    protected $accessProfileService;


    public function __construct(ZmartAccessProfileService $accessProfileService) {
        $this->accessProfileService = $accessProfileService;
    }

    public function find(DTO\Request $request, DTO\Response $response) {
        //  $conditions = $request->getFilterCriteria()->getConditions();
        //  $page = $request->getFilterCriteria()->getPage();
        //  $pageSize = $request->getFilterCriteria()->getPageSize();
        $params = $request->getParameters();
        $filter = new ZmartFilterCriteria($this->dataSourceName, array());
        // Search ALL
        if (array_key_exists("search_in", $params)) {
            $searchValues = $params["search_in"];
            $operation = FilterCriteria::EQ;
            $value = $params["search"];
            
            if(is_array($searchValues) && count($searchValues) > 0){
                $searchValues = (count($searchValues) > 1) ? implode('|',$searchValues) : $searchValues[0];
            }
            if(is_string($value)){
                $operation = is_array($searchValues) ? FilterCriteria::LIKE_ALL : FilterCriteria::LIKE_I;
                $value = "%".$value."%";
            } else if(is_array($value)){
                $operation = FilterCriteria::IN;
            }
            
            $filter->addCondition($searchValues, $operation, $value);
        }

        // Filter by field
        if (array_key_exists("in", $params)) {
            foreach ($params["in"] as $paramFilter){
                if(!array_key_exists($paramFilter, $params))
                    continue;
                $operator = FilterCriteria::EQ;
                $filterValue = $params[$paramFilter];
                if (is_string($filterValue)){
                    $filter->addCondition($paramFilter, FilterCriteria::LIKE_I, $filterValue);
                }
                else {
                    $filter->addCondition($paramFilter, $filterValue);
                }
            }
        }
        
        try {
            $accessProfile = $this->accessProfileService->find($filter->getConditions(), null, null);
            //  $response->addDataSet(new DataSet('zmartAccessProfile', $accessProfile));
            $this->sendResponse(Utilities::parseDataSourceZeedhiNext($accessProfile), 200);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }


}