<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class PeriodCompare extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_PERIODS_LIST_COMPARE = "574f230498ba51386fab2a81";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    public function fetchWidgetData(){
        $widget = $this->widget;
        $widgetDataService = InstanceManager::getInstance()->getService('\Service\WidgetData');
        $this->widgetData = $widgetDataService->fetchWidgetData($widget['_id'], $widget['parameters']['structureId'], null, $widget['parameters']['period'],
                $widget['parameters']['facts'], "",array(),null,null,$widget['parameters']['drillDownFilters']);
    }

    protected function getChangePercentage($value, $previousValue){
        $sign = $previousValue == $value ? "" : ($previousValue < $value ? "+" : "-");
        $number = $previousValue == 0 ? "∞%" : number_format(abs(100-($value/$previousValue*100)),2)."%";
        return ($previousValue === null) ? "" : ($sign.$number);
    }

    protected function getPeriodListSorted($widget){
        $periodList = [];
        foreach($widget["period"] as $period){
            $periodList[strtotime($period["start"])] = implode("_",[$period["start"], $period["end"]]);
        }
        ksort($periodList);
        return array_values($periodList);
    }

    public function getChildren(){
        $data = [];
        $widget = $this->widget;
        $periodList = $this->getPeriodListSorted($widget);
        $passedFirstRow = false;
        $dataSourceDataList = [];
        $dataId = 0;
        $mask = $widget["columns"][array_flip(array_column($widget["columns"], "name"))[$widget["periodsFact"]]]["mask"] ?? "";
        foreach($this->widgetData as $widgetData){
            $dataSourceData = [];
            $previousValue = null;
            foreach($periodList as $period){
                $index = $widget["periodsListFact"] . "_" . $period;
                $value = !empty($widgetData[$index]) ? $widgetData[$index] : 0;
                $change = $this->getChangePercentage($value, $previousValue);
                $dataSourceData[] = [
                    "period" => str_replace("_"," - ",$period),
                    "value" => strval($value),
                    "change" => $change,
                    "changeValue" => floatval($change),
                    "infinity" => ($previousValue == 0),
                    "label" => $widgetData["zmartCustomDimension"],
                    "id" => $dataId,
                ];
                $previousValue = $value;
                $dataId++;
            }
            $dataSourceDataList[] = $dataSourceData;
        }
        usort($dataSourceDataList, function($a, $b) use ($widget){
            $valueA = $a[count($a)-1]["changeValue"];
            $valueB = $b[count($b)-1]["changeValue"];
            if(strtolower($widget["zmartOrdenationComparePeriods"]) == "asc"){
                return $valueA == $valueB ? 0 : ($valueA > $valueB ? -1 : 1);
            }
            return $valueA == $valueB ? 0 : ($valueA < $valueB ? -1 : 1);
        });
        $index = 0;
        foreach($dataSourceDataList as $dataSourceData){
                $data[] = [
                    "name" => "row1_".$index,
                    "component" => "ZdRow",
                    "cssClass" => $passedFirstRow ? "zd-mt-6" : "",
                    "children" => [
                        [
                            "name" => "text_".$index,
                            "component" => "ZdText",
                            "text" => $dataSourceData[0]["label"],
                            "tag" => "h3",
                        ],
                    ],
                ];
                $data[] = [
                    "name" => "rowGrid_".$index,
                    "component" => "ZdRow",
                    "children" => [
                        [
                            "name" => "colGrid_".$index,
                            "component" => "ZdCol",
                            "children" => [
                                [
                                    "name" => "grid_".$index,
                                    "component" => "ZdGrid",
                                    "fillWidth" => true,
                                    "showTableHead" => false,
                                    "showFooter" => false,
                                    "showHeader" => false,
                                    "toolbarSlot" => [],
                                    "maskFunction" => $mask,
                                    "columns" => [
                                        [
                                            "name" => "period",
                                            "label" => "",
                                            "minWidth" => '200px',
                                            'align' => 'left',
                                        ],
                                        [
                                            "name" => "value",
                                            "label" => "",
                                            "minWidth" => '200px',
                                            'align' => 'center',
                                        ],
                                        [
                                            "name" => "changeIcon",
                                            "label" => "",
                                            "type" => "action",
                                            "minWidth" => '200px',
                                            'align' => 'center',
                                            "children" => [
                                                [
                                                    "name" => "changeAction_".$index,
                                                    "component" => "ZdButton",
                                                    "icon" => true,
                                                    "label" => "",
                                                    "iconName" => "",
                                                    "fab" => false,
                                                    "flat" => false,
                                                    "round" => false,
                                                    "conditions" => [
                                                        "isVisible" => "{{ChartViewController.periodCompareChangeActionVisibility}}",
                                                    ],
                                                ]
                                            ]
                                        ]
                                    ],
                                    "datasource" => [
                                        "uniqueKey" => "id",
                                        "data" => $dataSourceData
                                    ]
                                ]
                            ]
                        ]
                    ],
                ];
                $passedFirstRow = true;
                $index++;
        }
        return $data;
    }


    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "viewTypeId" => self::TYPE_ID_PERIODS_LIST_COMPARE,
            "component" => "zdContainer",
            "children" => $this->getChildren(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
