<?php

namespace Service;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;

class DynamicAggregate {

    public function convertFilterCriteriaToZmartFilterCriteria(FilterCriteria $filterCriteria) {
        $conditions = $filterCriteria->getConditions();
        $zmartFilterCriteria = new ZmartFilterCriteria($filterCriteria->getDataSourceName());
        foreach($conditions as $condition) {
            switch (true) {
                case $this->isGroupBy($condition):
                    $zmartFilterCriteria->setGroupBy($condition['name']);
                    break;
                case $this->isAggregateFunction($condition):
                    $zmartFilterCriteria->addAggregateFunction($this->getLabelOrName($condition), $condition['operator'], $condition['name']);
                    break;
                case $this->isOrderBy($condition):
                    $zmartFilterCriteria->addOrderBy($condition['name']['name'], $condition['name']['direction']);
                    break;
                default:
                    $zmartFilterCriteria->addCondition($condition['name'], $condition['operator'], $condition['value']);
            }
        }
        return $zmartFilterCriteria;
    }

    protected function isGroupBy($condition) {
        return $condition['operator'] === ZmartFilterCriteria::GROUP_BY;
    }

    protected function isAggregateFunction($condition) {
        $functions = array(
            ZmartFilterCriteria::SUM,
            ZmartFilterCriteria::AVG,
            ZmartFilterCriteria::MIN,
            ZmartFilterCriteria::MAX,
            ZmartFilterCriteria::COUNT
        );
        return in_array($condition['operator'], $functions);
    }

    protected function isOrderBy($condition) {
        return $condition['operator'] === ZmartFilterCriteria::ORDER_BY;
    }

    protected function getLabelOrName(array $condition) {
        return isset($condition['label']) ?:$condition['name'];
    }
} 