<?php

namespace Service;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use \Service\GoalsNotifications;

class Goals {

    protected $dataSourceName = 'goals';
    /** @var \Service\Manager\Mongo\Manager $mongoManager */
    protected $mongoManager;
    protected $goalsNotificationsService;

    public function __construct (Manager $mongoManager){
        $this->mongoManager = $mongoManager;
    }
    
     protected function findBy($propertyName, $operator, $propertyValue) {
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition($propertyName, $operator, $propertyValue);
        $filterCriteria->addCondition('isActive', "=", true);
        return $this->mongoManager->findBy($filterCriteria)->getRows();
    }

    public function getGoalsByDataSourceId($dataSourceId) {
        return $this->findBy('dataSourceId', "=", $dataSourceId);
    }

    public function getGoalsByViewId($viewId) {
        return $this->findBy('viewId', "=", $viewId);
    }

    public function getGoalsByIds(array $ids) {
        return $this->findBy('_id', FilterCriteria::IN, $ids);
    }
    
    public function getGoalById($goalId) {
        $goal = $this->findBy('_id', '=', $goalId);
		return count($goal) > 0 ? $goal[0] : null;
    }

    public function delete($goal) {
        $dataSet = new DataSet($this->dataSourceName, $goal);
        return $this->mongoManager->delete($dataSet);
    }
}