import { Http, Metadata, IDictionary } from '@zeedhi/core';
import { Col, Form } from '@zeedhi/common';
import { SkeletonMetadata } from '@/Services/SkeletonMetadata';
import { ChartViewService } from '../Services/ChartViewService';
import { ZmartChartsApex } from '../Services/Chats/ZmartChartsApex';

export class ChartViewController {

    public loadingTemplate = {
            "name": "result",
            "component": "ZdImage",
            'height': "100%",
            "src": "../img/loading3.gif"
        };

    public async apexChartOnCreated({ component }:any) {
        const urlParams = new URLSearchParams(window.location.search);
        const viewId = urlParams.get('name') ? urlParams.get('name') : '';
        if (typeof viewId === 'string') {
            this.buildSkeletonChart(component);
            await this.buildWidgetByViewId(component, viewId, {});
        }
    }

    public buildSkeletonChart(component: any) {
        component.children = [
            SkeletonMetadata.skeletonIndicator().children[0]
        ];
    }

    public async cardTitleOnMounted({ component }:any) {
        const urlParams = new URLSearchParams(window.location.search);
        const viewId = urlParams.get('name') ? urlParams.get('name') : '';
        if (viewId) {
            let element: Element | null = document.getElementById(viewId);
            while (element) {
                if (element.className === 'tek-card-title-text') {
                    element.setAttribute('style', component.cssStyle);
                    break;
                }
                element = element.children && element.children[0] ? element.children[0] : null;
            }
        }
    }

    public async buildWidgetByViewId(component: any, viewId: string, drillDownFilters: any) {
        if (!drillDownFilters) {
            drillDownFilters = {};
        }
        const params = {
            "requestType": "Row",
            "row": {
                "drillDownFilters": drillDownFilters,
                "serieColors": 0,
                "structureId": "",
                "structureName": "",
                "viewId": viewId
            }
        };
        const response = await Http.post("/buildWidgetByViewIdNext", params);
        const widget = response.data.dataset.widget;
        component.children = [];
        if (widget[0].name === 'notFound') {
            component.children = widget;
        } else {
            const charts = ZmartChartsApex.build(widget, this, component);
            const widgetChats = charts.getCharts();
            component.children = widgetChats;
        }
    }

    public static openEditView() {
        const colViewEdit = Metadata.getInstance<Col>('col-view-edit');
        colViewEdit.isVisible = true;
        colViewEdit.cols = 4;
        colViewEdit.sm = 4;
        colViewEdit.md = 4;
        colViewEdit.lg = 4;
        const colViewChart = Metadata.getInstance<Col>('col-view-chart');
        colViewChart.cols = 8;
        colViewChart.sm = 8;
        colViewChart.md = 8;
        colViewChart.lg = 8;
        ChartViewController.createFieldsEditView().then();
    }

    public static closeEditView() {
        const colViewEdit = Metadata.getInstance<Col>('col-view-edit');
        colViewEdit.isVisible = false;
        const colViewChart = Metadata.getInstance<Col>('col-view-chart');
        colViewChart.cols = 12;
        colViewChart.sm = 12;
        colViewChart.md = 12;
        colViewChart.lg = 12;
    }

    public static async createFieldsEditView() {
        const params = {
            "requestType": "FilterData",
            "filter": {}
        };
        const response = await Http.post("/viewProperty", params);
        if (response.data.dataset.viewProperty) {
            const colViewEdit = Metadata.getInstance<Col>('form-view-prop-edit');
            response.data.dataset.viewProperty.forEach(function (viewProperty: any) {
                if (viewProperty) {
                    viewProperty.component = "ZdTextInput";
                }
            });
            // colViewEdit.children = response.data.dataset.viewProperty;
        }
    }

    public static setFieldsEditView(typeView: any) {
        const colViewEdit = Metadata.getInstance<Col>('form-view-prop-edit');
        colViewEdit.children.forEach(function (field: any) {
            if (typeView.properties.indexOf(field._id) !== -1) {
                field.isVisible = true;
            } else {
                field.isVisible = false;
            }
        });
    }

    public periodCompareChangeActionVisibility({row, column}: any) {
        const button = column.children[0];
        const value = row.infinity ? Infinity : parseFloat(row.changeValue);
        if (value < 0) {
            button.color = 'error';
            button.iconName = 'mdi-chevron-down';
        } else if (value > 0) {
            button.color = 'success';
            button.iconName = 'mdi-chevron-up';
        } else {
            button.color = 'grey';
            button.iconName = '';
        }
        button.label = (value === Infinity ? '∞' : Math.abs(value)) + '%';
        return !!row.change;
    }
}
