import { Loader, Metadata } from '@zeedhi/core';
import { Modal, ModalService, DialogService, LoadingService, Container } from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { DataSourceService } from '@/Services/DataSourceService';
import { Utilities } from '@/Services/Utilities';
import { DataSourceController } from './DataSourceController';

export class DSImportHistoryController {
    private mainController: DataSourceController = Loader.getInstance("DataSourceController");
    public service = new DataSourceService();

    private get datasourceData() {
        return this.mainController.datasourceData;
    }

    private importHistoryModal!: Modal;
    public importHistoryData: any = {};
    public async openImportHistoryModalModal({ row }: any) {
        this.importHistoryModal?.destroy();
        this.importHistoryModal = await ModalService.createFromJson('dataSource/importHistory/importHistoryLog', true);
        this.importHistoryData = row || {};
        this.importHistoryModal.show();
    }
    public closeImportHistoryModal() {
        this.importHistoryModal.hide();
    }
    public importHistoryLogOnMounted({ component }: any) {
        const importHistoryLog = component.getChildInstance("importHistoryLog");
        importHistoryLog.value = this.importHistoryData.logs?.map((el: any) => `Type: ${el.type}\nMessage: ${el.message}`).join("\n\n");
    }
    public deleteImportHistory(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/importHistory/remove", "importHistory-grid");
    }
    public updateImportHistoryFilter() {
        if (Metadata.getInstances('importHistory-grid').length === 0 || !Metadata.getInstances('importHistory-grid')[0]) {
            return;
        }
        const dataSourceId = this.datasourceData._id;
        const importHistoryGrid: any = Metadata.getInstance<TekGrid>("importHistory-grid");
        importHistoryGrid.datasource.dynamicFilter = {
            dataSourceId: [
              {
                operation: "EQUALS",
                relation: "AND",
                value: dataSourceId,
              }
            ]
          };
        importHistoryGrid.datasource.get();
    }
}
